#!/bin/bash
#
# Simple Gallery, published under the GPL, 
# (C) by Hans Matzen, hans@hansmatzen.de, 2005, Germany
#

#
# default - parameter
#
version=0.5
# this is the installationdirectory of simplegal.sh
simplegaldir=/usr/local/src/simplegal/

#
# some useful programs
#
prg_convert=`which convert`
prg_identify=`which identify`
prg_getopt=`which getopt`

# keep these blank because they are mandatory
picdir=
galdir=
albums=

# defaults for the other parameters
thumbsize=128
pics_per_col=5
one_page_per_pic=0
show_name=0
show_geometry=0
show_size=0
show_creationdate=0
font="Helvetica"
fontsize=12
col_font="#000000"
col_back="#FFFFFF"
col_border="#0000FF"
border_size=1
index_title="simplegal.sh Image Gallery"

#
# help function prints program information
#
function help()
{
    echo "Version: simplegal.sh $version http://www.hansmatzen.de/simplegal"
    echo "Published under the GPL by Hans Matzen <hans@hansmatzen.de>, 2005"
    echo 
    echo "Usage: simplegal.sh [options ...] imagedir gallerydir [albums]"
    echo "  imagedir    - the driectory where the original images reside"
    echo "  gallerydir  - the directory where the gallery will be build" 
    echo "  albums      - select specific subdirs from imagedir" 
    echo
    echo "Where options include:"
    echo "  -background color   use background color"
    echo "  -border color       use bordercolor color"
    echo "  -borderwidth count  draw border around images with width count"
    echo "  -columnwidth count  one row will contain count images"
    echo "  -creation           show creationdate at bottom of pages"
    echo "  -font fontname      use font fontname"
    echo "  -fontsize size      set size of font to size"
    echo "  -foreground color   use fontcolor color"
    echo "  -geometry           show geometry of image in index.html"
    echo "  -name               show image name in index.html"
    echo "  -onepage            create one html page per image"
    echo "  -size               show size of image in index.html"
    echo "  -thumbsize height   create thumbnails with height pixels"
    echo "  -title title        use title as a page titel and heading"
    echo "  -version            prints version and exits"
}

#
# prints version information
#
function version()
{
    echo "simplegal.sh $version"
}


#
# print the current parameter settings
#
function show_parms() {
    echo "simplegal.sh - Parameters"
    echo "Simplegaldir: $simplegaldir"
    echo "Imagedir: $picdir"
    echo "Gallerydir: $galdir"
    echo "Albums: $albums"
    echo "--thumbsize $thumbsize"
    echo "--columnwidth $pics_per_col"
    echo "--onepage $one_page_per_pic"
    echo "--name $show_name"
    echo "--geometry $show_geometry"
    echo "--size $show_size"
    echo "--creation $show_creationdate"
    echo "--font $font"
    echo "--fontsize $fontsize"
    echo "--foreground $col_font"
    echo "--background $col_back"
    echo "--border $col_border"
    echo "--borderwidth $border_size"
    echo "--title $index_title"
}

#
#
#   MAIN
#
#

#
# pars the commandline
#
opts=`$prg_getopt -o Vh -a --long background:,border:,borderwidth:,columnwidth:,creation,font:,fontsize:,foreground:,geometry,help,name,onepage,size,thumbsize:,title:,version -- "$@"`
# everything ok with getopt?
if [ $? != 0 ] ; then echo "Terminating..." >&2 ; exit 1 ; fi
# move the values to bashs arg
eval set -- "$opts"
# step through
while true ; do
    case "$1" in
	-V|--version)          version;shift;;
	-h|--help)             help;shift;;
	--background)          col_back="$2"; shift 2 ;;
	--border)              col_border="$2";shift 2;; 
	--borderwidth)         border_size="$2";shift 2;;   
	--columnwidth)         pics_per_col="$2";shift 2;;
	--creation)            show_creationdate=1;shift;;
	--font)                font="$2";shift 2;;
	--fontsize)            fontsize="$2";shift 2;;
	--foreground)          col_font="$2";shift 2;;
	--geometry)            show_geometry=1;shift;;
	--name)                show_name=1;shift;;
	--onepage)             one_page_per_pic=1;shift;;
	--size)                show_size=1;shift;;
	--thumbsize)           thumbsize="$2";shift 2;;
	--title)               index_title="$2";shift 2;;
	# example for optional parameter
               #	-c|--c-long) 
               # c has an optional argument. As we are in quoted mode,
               # an empty parameter will be generated if its optional
               # argument is not found.
	       #case "$2" in
	       #	"") echo "Option c, no argument"; shift 2 ;;
	       #	*)  echo "Option c, argument \`$2'" ; shift 2 ;;
	       #    esac ;;
	--) shift ; break ;;
	*) echo "Internal error!" ; exit 1 ;;
    esac
done
# get mandatory parameters
picdir=$1;shift
galdir=$1;shift
albums=$@;shift

# check if we have what we need: the two directories
if [ "$picdir" == "" ]; then
    help; exit 1;
fi
if [ "$galdir" == "" ]; then
    help; exit 1;
fi

# if we got no albums take it all
if [ "$albums" == "" ]; then
    for i in $picdir/*; do 
	if [ -d $i ]; then
	    albums="$albums `basename $i`"
	fi
    done
fi

# print the parameters out
show_parms
# ask if we should continue
while true; do
    answer=
    echo -n "Please check the parameters. Continue? (n/y)"
    read answer
    if [ "$answer" == "n" ]; then
	exit 1;
    fi
     if [ "$answer" == "y" ]; then
	break;
    fi
done

#
# get current date
#
if [ $show_creationdate == 1 ]; then
    currdate="am `date +"%A, %d.%B %Y"`"
else
    currdate=""
fi
#
# create gallerydir if necessary
#
echo -n Creating gallery directory $galdir ...
if [ ! -d $galdir ]; then
   mkdirhier $galdir
fi
echo done.
cd $galdir

#
# removing main index page
#
rm -f index.html

#
# creating sgal.css
#
cat >>sgal.css <<EOF 
/* CSS for simplegal.sh */
BODY {
  color: $col_font; 
  background: $col_back;
  font-family: $font, sans-serif;
  font-size: $font_size pt; 
  margin: 4%; 
}

H1 {
  color: $col_font;
}

TABLE {
  text-align: center; 
  margin-left: auto; 
  margin-right: auto;
}

TD { 
  color: #000000; 
  padding: 1em
}

IMG.photo { 
  border: $border_size px solid $col_border; 
}

a:link    { color: #7f7f7f; }
a:visited { color: #7f7f7f; }
EOF

#
# repeat for each album the creation sequence
#
for k in $albums; do
    #
    # remove old files
    #
    echo -n "Removing old files for album $k..."
    if [ $k ]; then
	rm -rf $k/*
    fi
    echo "done."

    echo Now creating galery for album $k ...
    # creating dirs
    echo -ne "\tCreating directories for album $k ..."
    mkdirhier $k/html
    mkdirhier $k/thumbs
    mkdirhier $k/images
    echo done.
   
    # copying nav icons and other template files
    echo -ne "\tCopying auxiliary files..."
    cp $simplegaldir/next.png $simplegaldir/back.png $k
    cp $simplegaldir/gohome.png $simplegaldir/up.png $k
    cp $simplegaldir/valid-html401.png $k
    cp $simplegaldir/valid-html401.png $galdir
    echo done.

    # copying original images
    echo -ne "\tCopying original Images for album $k ..."
    cp $picdir/$k/* $k/images
    echo done.

    # creating thumbnails
    echo -ne "\tCreating thumbnails..."
    let img_count=0
    for j in $k/images/*; do
	fname=`echo $j|cut -d"/" -f3` 
	$prg_convert $j  -geometry $thumbsize $k/thumbs/$fname
	let img_count=$img_count+1
    done
    echo done.

    # creating html pages
    if [ $one_page_per_pic == 1 ]; then
	echo -ne "\tCreating single image pages..."
	prevfname=""
	nextfname=""
	fname=""
	for j in $k/thumbs/* END; do
	    # update the three image names for navigation
	    oldfname=$fname
	    fname=$nextfname
	    nextfname=`echo $j|cut -d"/" -f3`

	    if [ $fname ]; then
		cat >>$k/html/$fname.html <<EOF 
<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN" "http://www.w3.org/TR/html4/loose.dtd">
<html>
<head>
<meta http-equiv="content-type" content="text/html; charset=utf-8">
<meta name="Generator" content="simplegal.sh v0.1">
<meta name="date" content="$currdate">
<title>Image Gallery 2005 / 1 : $fname</title>
<link rel="stylesheet" type="text/css" href="../../sgal.css">
</head>
<body>
<div align="center">
EOF
		if [ "$oldfname" != "" ]; then
		    cat >>$k/html/$fname.html <<EOF
<a href="$oldfname.html"><img src="../back.png"></a>
EOF
		fi
		cat >>$k/html/$fname.html <<EOF
&nbsp; | &nbsp;
<a href="../index.html"><img src="../up.png" border="0" alt="Albumindex"></a>
&nbsp; | &nbsp;<a href="../../index.html"><img src="../gohome.png" border="0" alt="Albenliste"></a>&nbsp; | &nbsp;
EOF
		if [ "$nextfname" != "END" ]; then
		    cat >>$k/html/$fname.html <<EOF
<a href="$nextfname.html"><img src="../next.png"></a>
EOF
		fi
		cat >>$k/html/$fname.html <<EOF
<br><hr>    
<br>
<img class="photo" src="../images/$fname" alt="$fname"><br><br></div>
<hr>
<div>
<img src="../valid-html401.png" alt="HTML 4.01" height="31" width="88"/>Diese Seite wurde mit simplegal.sh $currdate erstellt.
</div>
</body>
</html>
EOF
	    fi
	done
	echo "done."
    fi
   
    # creating index.html for album
    echo -ne "\tCreating index.html for album $k ..."
    #
    # remove old file if it exists
    #
    rm -f $k/index.html
    #
    # write header
    #
    cat >>$k/index.html <<EOF 
<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01//EN" "http://www.w3.org/TR/html4/strict.dtd">
<html>
<head>
<meta http-equiv="content-type" content="text/html; charset=utf-8">
<meta name="Generator" content="simplegal.sh v0.1">
<meta name="date" content="$currdate">
<title>$index_title (Album: $k)</title>
<link rel="stylesheet" type="text/css" href="../sgal.css">
</head>
<body>
<p>
<a href="../index.html"><img src="gohome.png" border="0" alt="Albenliste">
</a></p>
<h1>Bildergalerie f&uuml;r Album&quot; $k &quot;</h1>
<table width="100%" border=1 cellpadding=0 cellspacing=0 style="page-break-before: always">
<col width="20%"><col width="80%">
<tr valign=top><td align=left>
<i>Bilder:</i>
</td><td align=left>

$img_count

</td></tr></table>
<table>
EOF
    let rowcount=0
    echo "<tr>" >> $k/index.html
    for j in $k/thumbs/*; do 
        #
        # calc parts of path
        #
	fname=`echo $j|cut -d"/" -f2`
	ofname=`echo $j|cut -d"/" -f3` 
        #
        # calc size and geometry of the image
        #
	id=`$prg_identify $k/images/$ofname`
	size=`echo $id|cut -d" " -f5`
	geometry=`echo $id|cut -d" " -f3`
        #
        # check for row break
        #
	if [ "$rowcount" == "$pics_per_col" ]; then
	    echo "</tr><tr>" >> $k/index.html
	    let rowcount=0
	fi
        #
        # check how we must link to the image or to the single-page 
        #
	if [ $one_page_per_pic == 1 ]; then
	    cat >>$k/index.html <<EOF 
<td align='center'>
<a href="html/$ofname.html"><img class="photo" src="$fname/$ofname" alt="$fname/$ofname"></a>
EOF
	else
	    cat >>$k/index.html <<EOF 
<td align='center'>
<a href="images/$ofname"><img class="photo" src="$fname/$ofname" alt="$fname/$ofname"></a>
EOF
	fi
	if [ $show_name == 1 ]; then
	    echo "<div>$ofname</div>" >>$k/index.html
	fi
	if [ $show_geometry == 1 ]; then
	    echo "<div>$geometry</div>" >>$k/index.html
	fi
	if [ $show_size == 1 ]; then
	    echo "<div>($size)</div>" >>$k/index.html
	fi
	echo "</td>" >>$k/index.html
       
	let rowcount=$rowcount+1
    done
    #
    # footer output
    #
    echo "</tr></table>" >> $k/index.html
    cat >>$k/index.html <<EOF 
<hr>
<p>
<img src="../valid-html401.png" alt="HTML 4.01" height="31" width="88"/>Diese Seite wurde mit simplegal.sh $currdate erstellt.
</p>
</body>
</html>
EOF
    echo "done."
    echo "done."
done

#
# finally create the album index
#
echo -n "Creating main index page..."
#
# write header
#
   cat >>index.html <<EOF 
<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01//EN" "http://www.w3.org/TR/html4/strict.dtd">
<html>
<head>
<meta http-equiv="content-type" content="text/html; charset=utf-8">
<meta name="Generator" content="simplegal.sh v0.1">
<meta name="date" content="$currdate">
<title>$index_title</title>
<link rel="stylesheet" type="text/css" href="sgal.css">
</head>
<body>
<h1>$index_title</h1><p>

<i>Albenliste:</i><br>
<hr>
<p> 
<table><tr>
EOF

#
# write album rows
#
let rowcount=0
for k in $albums; do
    firstpic=`ls $k/images|tail -n1`
    numpics=`ls $k/images|grep -c ""`
    #
    # check for row break
    #
    if [ "$rowcount" == "$pics_per_col" ]; then
	echo "</tr><tr>" >> index.html
	let rowcount=0
    fi
cat >>index.html <<EOF 
<td><a href="$k/index.html">
<img class="photo" src="$k/thumbs/$firstpic" alt="$k/thumbs/$firstpic"></a><br>
<a href="$k/index.html">$k</a><br> [ $numpics Bilder ]<br></td>
EOF
let rowcount=$rowcount+1 
done

#
# write footer 
#
   cat >>index.html <<EOF 
</tr></table>
</p>
<hr>
<p>
<img src="valid-html401.png" alt="HTML 4.01" height="31" width="88"/>Diese Seite wurde mit simplegal.sh $currdate erstellt.
</p>
</body>
</html>
EOF
echo "done."
echo "SimpleImageGallery created."
